﻿/*
 * TCPEchoClient1
 *
 * Author Michael Claudius, ZIBAT Computer Science
 * Version 1.0. 2014.02.12, 1.1 2015.10.09
 * Copyright 2014 by Michael Claudius
 * Revised 2015.11.10
 * All rights reserved
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace SSLClient
{
    class SSLEchoClient
    {
        static void Main(string[] args)
        {
            //Client certificate
            string clientCertificateFile = "c:/certificates/ClientSSL.cer";
            X509Certificate clientCertificate = new X509Certificate(clientCertificateFile, "secret");

            //remember there is no revocation list
            bool checkCertificateRevocation = false;  //if true the remote revocation chain will be checked and leads to an error

            SslProtocols enabledSSLProtocols = SslProtocols.Tls;  //superseeds the former SslProtocols.Ssl3

            RemoteCertificateValidationCallback userCertificateValidationCallback;
            LocalCertificateSelectionCallback localCertificateCallback;
                      
            //Alternative for validation of client
            X509CertificateCollection certificateCollection = new X509CertificateCollection();
            certificateCollection.Add(clientCertificate);

            string serverName = "FakeServerName";
            bool leaveInnerStreamOpen = false;

            TcpClient clientSocket = new TcpClient("localhost", 6789);
            Stream unsecureStream = clientSocket.GetStream();
            //No revocation
            //SslStream sslStream = new SslStream(unsecureStream, leaveInnerStreamOpen, new RemoteCertificateValidationCallback(ValidateServerCertificate), null);

            //Setup for handling the validation of server  
            userCertificateValidationCallback = new RemoteCertificateValidationCallback(ValidateServerCertificate);
            localCertificateCallback = new LocalCertificateSelectionCallback(CertificateSelectionCallback);

            SslStream sslStream = new SslStream(unsecureStream, leaveInnerStreamOpen, userCertificateValidationCallback, localCertificateCallback);
            sslStream.AuthenticateAsClient(serverName, certificateCollection, enabledSSLProtocols, checkCertificateRevocation);

            StreamReader sr = new StreamReader(sslStream);
            StreamWriter sw = new StreamWriter(sslStream);
            sw.AutoFlush = true; // enable automatic flushing
            Console.WriteLine("Client authenticated");
            for (int i = 0; i < 5; i++)
            {
                string message = Console.ReadLine();
                sw.WriteLine(message);
                string serverAnswer = sr.ReadLine();
                Console.WriteLine("Server: " + serverAnswer);
            }
            sslStream.Close();
            clientSocket.Close();
        }

        private static bool ValidateServerCertificate(object sender, X509Certificate serverCertificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            Console.WriteLine("Client Sender: " + sender.ToString());
            Console.WriteLine("Client server certificate : " + serverCertificate.ToString());
            Console.WriteLine("Client Policy errors: " + sslPolicyErrors.ToString());

            if (sslPolicyErrors == SslPolicyErrors.None)
            {
                Console.WriteLine("Client validation of server certificate successful.");
                return true;
            }
            Console.WriteLine("Errors in certificate validation:");
            Console.WriteLine(sslPolicyErrors);
            return false;
        }

        private static X509Certificate CertificateSelectionCallback(object sender, string targetHost, X509CertificateCollection localCollection,
                                                                   X509Certificate remoteCertificate, string[] acceptableIssuers)
        {

            return localCollection[0];
        }
    }
}
