﻿/*
 * SSLEchoServer
 *
 * Author Michael Claudius, ZIBAT Computer Science
 * Version 1.0. 2014.02.12, 1.1 2015.10.09
 * Copyright 2014 by Michael Claudius
 * Revised 2015.11.10, 2017.10.31
 * All rights reserved
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;

// Inspired by https://msdn.microsoft.com/en-us/library/system.net.security.sslstream.aspx

namespace SSLEchoServer
{
    class SSLEchoServer
    {

        public static void Main(string[] args)
        {
            string serverCertificateFile = "c:/certificates/ServerSSL.cer";  // or ServerSSL.pfx
            X509Certificate serverCertificate = new X509Certificate(serverCertificateFile, "secret");

            bool clientCertificateRequired = true;
            bool checkCertificateRevocation = false; ////if true the remote revocation chain will be checked and leads to an error
            SslProtocols enabledSSLProtocols = SslProtocols.Tls;  //superseeds the former SslProtocols.Ssl3

            RemoteCertificateValidationCallback userCertificateValidationCallback;
            LocalCertificateSelectionCallback localCertificateCallback;

            TcpListener serverSocket = new TcpListener(6789);
            serverSocket.Start();
            Console.WriteLine("Server started");

            TcpClient connectionSocket = serverSocket.AcceptTcpClient();
            Console.WriteLine("Server activated");

            Stream unsecureStream = connectionSocket.GetStream();
            bool leaveInnerStreamOpen = false; //i.e. close the unsecureStream when sslStream is closed

            //Setup for handling the validation of client  
            userCertificateValidationCallback = new RemoteCertificateValidationCallback(ValidateClientCertificate);
            localCertificateCallback = null; // new LocalCertificateSelectionCallback(CertificateSelectionCallback);

            // Decorator/Wrapper design pattern  
            SslStream sslStream = new SslStream(unsecureStream, leaveInnerStreamOpen, userCertificateValidationCallback, localCertificateCallback);
            sslStream.AuthenticateAsServer(serverCertificate, clientCertificateRequired, enabledSSLProtocols, checkCertificateRevocation);

            Console.WriteLine("Server authenticated");

            StreamReader sr = new StreamReader(sslStream);
            StreamWriter sw = new StreamWriter(sslStream);
            sw.AutoFlush = true; // enable automatic flushing

            string message = sr.ReadLine();
            string answer = "";
            while (message != null && message != "")
            {
                Console.WriteLine("Client: " + message);
                answer = message.ToUpper();
                sw.WriteLine(answer);
                message = sr.ReadLine();
            }

            sslStream.Close();
            connectionSocket.Close();
            serverSocket.Stop();

        }

        private static bool ValidateClientCertificate(object sender, X509Certificate clientCertificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            Console.WriteLine("Server Sender: " + sender.ToString());
            Console.WriteLine("Server : " + clientCertificate.ToString());
            Console.WriteLine("Server : " + sslPolicyErrors.ToString());

            if (sslPolicyErrors == SslPolicyErrors.None)
            {
                Console.WriteLine("Server  validation of client certificate successful.");
                return true;
            }
            Console.WriteLine("Errors in certificate validation:");
            Console.WriteLine(sslPolicyErrors);
            return false;
        }

    }
}


